package protocol;

import java.io.*;
import java.nio.channels.*;
import java.net.*;
import java.util.*;

import message.*;
import component.*;


public class TcpClientTask extends Thread {
	
	private final Component c;
	
	private Selector selector;
	
	public TcpClientTask(Component c) {
		this.c = c;
		
    	try {
    		selector = Selector.open();
    	} catch (IOException e) {
			e.printStackTrace();
		}
	}
		
	private boolean addConnection() {

		synchronized (c.client) {

			int aSize = c.addressQueue.size();
			if (c.addressIndex == aSize)
				return false;

			InetSocketAddress address = (InetSocketAddress) c.addressQueue
					.get(c.addressIndex);
			
			c.addressIndex++;
			
			///*
			InetAddress ip = address.getAddress();
			int nSize = c.neighborList.size();
			for (int i = 0; i < nSize; i++) {
				Neighbor n = (Neighbor) c.neighborList.get(i);
				if (ip.equals(n.address.getAddress()))
					return true;
			}
			//*/
			
			// System.out.println("try to connect to " +
			// address.getAddress().toString() + ": " + address.getPort());

			try {
				// һ׽ͨ
				SocketChannel client = SocketChannel.open();
				// ͨʹ÷ģʽ
				client.configureBlocking(false);
				// עϵ¼
				client.register(selector, SelectionKey.OP_CONNECT);
				// Ӵͨ׽
				client.connect(address);
				
			} catch (IOException e) {
				e.printStackTrace();
			}
			return true;
		}
	}
	
	private void addNeighbor(SocketChannel client) {
		
		synchronized (c.client) {

			InetSocketAddress address = (InetSocketAddress) client
					.socket().getRemoteSocketAddress();

			Neighbor n = new Neighbor(c.fileInfo.pieceNum, address);
			n.isOutgoing = true;

			System.out.println("generate a handshake...");
			byte[] infoHash = c.fileInfo.infoHash;
			byte[] peerId = c.client.peerId.getBytes();

			byte[] context = new byte[40];
			System.arraycopy(infoHash, 0, context, 0, 20);
			System.arraycopy(peerId, 0, context, 20, 20);

			Handshake hs = new Handshake(context);
			n.sendingQueue.addLast(hs);

			c.neighborList.add(n);
			
			//System.out.println("add a neighbor....");
		}
	}
	
    public void run() {

		while (true) {

			boolean hasNew = addConnection();

			if (!hasNew) {
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}	
			
			try {
				int numKeys = selector.select(50);
				if (numKeys == 0)
					continue;
			} catch (IOException e) {
				e.printStackTrace();
				continue;
			}	

			Set readyKeys = selector.selectedKeys();
			Iterator iter = readyKeys.iterator();
			
			while (iter.hasNext()) {

				SelectionKey key = (SelectionKey) iter.next();
				iter.remove();

				SocketChannel client = (SocketChannel) key.channel();
				
				if (!key.isConnectable())
					continue;
				
				try {
					if (client.isConnectionPending())
						client.finishConnect();
				} catch (IOException e) {
					e.printStackTrace();
					continue;
				}

				try {
					c.readers.put(client);
					c.writers.put(client);
				} catch (Exception e) {
					e.printStackTrace();
				}

				addNeighbor(client);
			}
		}
	}
}
