package protocol;

import java.nio.channels.*;
import java.util.*;
import java.net.*;
import java.io.*;

import component.*;

public class TcpServerTask extends Thread {

    private ServerSocketChannel server;
    
    private final Component c;
    
    private Selector selector;

    public TcpServerTask (Component c) {
        
    	this.c = c;
    	
    	try {
			// open a non-blocking server socket channel
			this.server = ServerSocketChannel.open();
			this.server.configureBlocking(false);

			// bind to localhost on designated port
			InetAddress addr = InetAddress.getLocalHost();
			//System.out.println("binding to address: " + addr.getHostAddress());
			this.server.socket().bind(new InetSocketAddress(addr, c.client.port));

			// get a selector
			this.selector = Selector.open();

			// register the channel with the selector to handle accepts
			this.server.register(selector, SelectionKey.OP_ACCEPT);
			
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(1);
		}
    }

    public void run() {

		while (true) {

			try {
				int numKeys = selector.select();
				if (numKeys == 0)
					continue;
			} catch (IOException e) {
				e.printStackTrace();
				continue;
			}

			// fetch the keys
			Set readyKeys = selector.selectedKeys();

			// run through the keys and process
			Iterator iter = readyKeys.iterator();
			_OUT: while (iter.hasNext()) {

				SelectionKey key = (SelectionKey) iter.next();
				iter.remove();

				ServerSocketChannel server = (ServerSocketChannel) key
						.channel();

				try {
					
					SocketChannel client = server.accept();
					InetSocketAddress address = (InetSocketAddress) client
								.socket().getRemoteSocketAddress();
					
					synchronized (c.client) {
						
						// Ѿдھӣܾ
						///*
						InetAddress ip = address.getAddress();
						int size = c.neighborList.size();
						for (int i = 0; i < size; i++) {
							Neighbor n = (Neighbor) c.neighborList.get(i);
							if (ip.equals(n.address.getAddress())) {
								key.cancel();
								client.close();
								continue _OUT;
							}	
						}
						//*/

						// һµھ
						Neighbor neighbor = new Neighbor(c.fileInfo.pieceNum,
								address);
						c.neighborList.add(neighbor);

					}
					
					c.readers.put(client);
					c.writers.put(client);
					
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
	}
}