package protocol;

import java.util.ArrayList;
import java.io.IOException;

import component.*;

public class WriteFileTask implements Runnable {

	private final Component c;

	public WriteFileTask(Component c) {
		this.c = c;
	}

	private final void writeFile() throws IOException {
		
		ArrayList queue = getPiece();
		int size = queue.size();
		for(int i = 0; i < size; i++) {
			Piece p = (Piece) queue.get(i);
			writePiece(p);
		}
		queue.clear();
	}
	
	private final ArrayList getPiece() {
		
		ArrayList queue = new ArrayList();
		synchronized (c.writingQueue) {
			
			int wSize = c.writingQueue.size();
			for (int i = 0; i < wSize; i++) {
				Piece p = (Piece) c.writingQueue.get(i);
				
				if (!p.isPieceCompleted())
					continue;
				
				//System.out.println("write piece to disk...");
				queue.add(p);
			}
			
			int qSize = queue.size();
			for(int i = 0; i < qSize; i++) {
				Piece p = (Piece) queue.get(i);
				c.writingQueue.remove(p);
			}	
		}
		return queue;
	}

	private final void writePiece(Piece p) throws IOException {

		synchronized (c.randomAccessFile) {

			int startPos = p.pieceIndex * c.fileInfo.pieceLen;
			int pieceLen = p.pieceContext.length;
			int endPos = startPos + pieceLen;
			int leftLen = pieceLen; // ʣpieceȡ

			int currentPos = 0; // Ѿλļλ
			int nextPos = 0; // һζλļλ

			int fileNum = c.fileInfo.lenList.length;
			for (int j = 0; j < fileNum; j++) {
				nextPos += c.fileInfo.lenList[j];
				if (startPos < nextPos) {
					if (endPos > nextPos) { // ֱʼַָļ,ʼд
						c.randomAccessFile[j].seek(startPos - currentPos);
						c.randomAccessFile[j].write(p.pieceContext, pieceLen - leftLen,
								nextPos - startPos);
						leftLen = startPos + leftLen - nextPos;
						startPos = nextPos;
					} else { // Ҫļȫļ
						c.randomAccessFile[j].seek(startPos - currentPos);
						c.randomAccessFile[j].write(p.pieceContext, pieceLen - leftLen,
								leftLen);
						break;
					}
				}
				currentPos += c.fileInfo.lenList[j];
			}
		}
	}

	public void run() {
		
		System.out.println("execute the write file task...");
		try {
			writeFile();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}
