package protocol;

import java.net.InetSocketAddress;
import java.nio.*;
import java.nio.channels.*;
import java.io.*;
import java.util.*;

import component.*;
import message.*;

public class WriteMessageTask extends Thread {
	
	private Component c;
    private Selector selector;
    
    public WriteMessageTask(Component c) {

		this.c = c;
		
		try {

			this.selector = Selector.open();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
		
	private void writeMsg() {

		try {
			int numKeys = selector.select(30);
			if (numKeys == 0)
				return;
		} catch (IOException e) {
			e.printStackTrace();
			return;
		}

		Set readyKeys = selector.selectedKeys();
		Iterator iter = readyKeys.iterator();

		while (iter.hasNext()) {

			SelectionKey key = (SelectionKey) iter.next();
			iter.remove();

			SocketChannel client = (SocketChannel) key.channel();

			InetSocketAddress address = (InetSocketAddress) client.socket()
					.getRemoteSocketAddress();

			ByteBuffer buffer = getWriteBuffer(address);

			if (buffer == null)
				continue;

			try {
				client.write(buffer);
			} catch (IOException e) {
				e.printStackTrace();
				key.cancel();
			}
		}
	}
	
	private ByteBuffer getWriteBuffer (InetSocketAddress address) {
		
		synchronized (c.client) {

			int size = c.neighborList.size();
			for (int i = 0; i < size; i++) {
				Neighbor n = (Neighbor) c.neighborList.get(i);
				if (!n.address.equals(address))
					continue;
				
				if (n.sendingQueue.isEmpty())
					return null;
				
				Message msg = (Message) n.sendingQueue.getFirst();
				
				ByteBuffer buffer = ByteBuffer.wrap(msg.msgContext);
				n.sendingQueue.removeFirst();
				
				return buffer;
			}
			
			return null;
		}
	}
	
	
    private void registerKey() {

		while (!c.writers.isEmpty()) {

			SocketChannel client = (SocketChannel) c.writers.poll();
			try {
				client.configureBlocking(false);
				client.register(selector, SelectionKey.OP_WRITE);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
    
    public void run() {

		while (true) {
			
			registerKey();
			writeMsg();
			
			/*
			try {
				Thread.sleep(30);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			//*/
		}
	}
}
