package bencode;

/*
 * Created on 2 juil. 2003
 * Copyright (C) 2003, 2004, 2005, 2006 Aelitis, All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * AELITIS, SAS au capital de 46,603.30 euros
 * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
 *
 */

/**
 * @author Olivier
 * 
 */

import java.nio.ByteBuffer;

public class ByteFormatter {
	final static char[] HEXDIGITS = { '0', '1', '2', '3', '4', '5', '6', '7',
			'8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };

	public static String nicePrint(String str) {
		return (nicePrint(str.getBytes(), true));
	}

	public static String nicePrint(byte[] data) {
		return (nicePrint(data, false));
	}

	public static String nicePrint(byte[] data, int max) {
		return (nicePrint(data, false, max));
	}

	public static String nicePrint(ByteBuffer data) {
		byte[] raw = new byte[data.limit()];

		for (int i = 0; i < raw.length; i++) {
			raw[i] = data.get(i);
		}

		return nicePrint(raw);
	}

	public static String nicePrint(byte[] data, boolean tight) {
		return (nicePrint(data, tight, 1024));
	}

	public static String nicePrint(byte[] data, boolean tight, int max_length) {
		if (data == null) {
			return "";
		}

		int dataLength = data.length;

		if (dataLength > max_length) {
			dataLength = max_length;
		}

		int size = dataLength * 2;
		if (!tight) {
			size += (dataLength - 1) / 4;
		}

		char[] out = new char[size];

		try {
			int pos = 0;
			for (int i = 0; i < dataLength; i++) {
				if ((!tight) && (i % 4 == 0) && i > 0) {
					out[pos++] = ' ';
				}

				out[pos++] = HEXDIGITS[(byte) ((data[i] >> 4) & 0xF)];
				out[pos++] = HEXDIGITS[(byte) (data[i] & 0xF)];
			}

		} catch (Exception e) {
			//Debug.printStackTrace(e);
		}

		try {
			return new String(out) + (data.length > max_length ? "..." : "");
		} catch (Exception e) {
			//Debug.printStackTrace(e);
		}

		return "";
	}

	public static String nicePrint(byte b) {
		byte b1 = (byte) ((b >> 4) & 0x0000000F);
		byte b2 = (byte) (b & 0x0000000F);
		return nicePrint2(b1) + nicePrint2(b2);
	}

	public static String nicePrint2(byte b) {
		String out = "";
		switch (b) {
		case 0:
			out = "0";
			break;
		case 1:
			out = "1";
			break;
		case 2:
			out = "2";
			break;
		case 3:
			out = "3";
			break;
		case 4:
			out = "4";
			break;
		case 5:
			out = "5";
			break;
		case 6:
			out = "6";
			break;
		case 7:
			out = "7";
			break;
		case 8:
			out = "8";
			break;
		case 9:
			out = "9";
			break;
		case 10:
			out = "A";
			break;
		case 11:
			out = "B";
			break;
		case 12:
			out = "C";
			break;
		case 13:
			out = "D";
			break;
		case 14:
			out = "E";
			break;
		case 15:
			out = "F";
			break;
		}
		return out;
	}

	public static String encodeString(byte[] bytes) {
		return (nicePrint(bytes, true));
	}

	public static String encodeString(byte[] bytes, int offset, int len) {
		byte[] x = new byte[len];

		System.arraycopy(bytes, offset, x, 0, len);

		return (nicePrint(x, true));
	}

	public static byte[] decodeString(String str) {
		char[] chars = str.toCharArray();

		int chars_length = chars.length - chars.length % 2;

		byte[] res = new byte[chars_length / 2];

		for (int i = 0; i < chars_length; i += 2) {

			String b = new String(chars, i, 2);

			res[i / 2] = (byte) Integer.parseInt(b, 16);
		}

		return (res);
	}

	/**
	 * Convert a Network Byte Order byte array into an int
	 *  
	 * @param array
	 * @return
	 *
	 * @since 3.0.1.5
	 */
	public static int byteArrayToInt(byte[] array) {
		return (array[0] << 24) & 0xff000000 | (array[1] << 16) & 0x00ff0000
				| (array[2] << 8) & 0x0000ff00 | array[3] & 0x000000ff;
	}

	//	4ֽתΪIp
	public String byteArrayToIp(byte[] bytedata, int starti) {
		String ip = "";
		int t = 0;
		for (int i = 0; i < 4; i++) {
			t = (short) bytedata[starti + i];
			if (t < 0)
				t = t + 256;
			ip += t + ".";
		}
		return ip.substring(0, ip.length() - 1);
	}

	//2ֽתΪint
	public int byteArrayToPort(byte[] b, int starti) {
		int s = 0;
		//    	System.out.println("starti+0:" + byteToHexString(b[starti + 0]) + " starti+1:" + byteToHexString(b[starti + 1]));
		if (b[starti + 0] >= 0) {
			s = s + b[starti + 0];
		} else {
			s = s + 256 + b[starti + 0];
		}
		s = s * 256;
		if (b[starti + 1] >= 0) {
			s = s + b[starti + 1];
		} else {
			s = s + 256 + b[starti + 1];
		}
		return s;
	}

	// 4ֽתΪ
	public int byteArrayToInt(byte[] bytedata, int i) {
		return ((bytedata[i] & 0xff) << 24) | ((bytedata[i + 1] & 0xff) << 16)
				| ((bytedata[i + 2] & 0xff) << 8) | (bytedata[i + 3] & 0xff);
	}

	// תΪ4ֽ
	public void intToByteArray(int intValue, byte[] byteData, int i) {
		byteData[i] = (byte) (intValue >>> 24);
		byteData[i + 1] = (byte) (intValue >>> 16);
		byteData[i + 2] = (byte) (intValue >>> 8);
		byteData[i + 3] = (byte) intValue;
	}

	// ֽתΪʮַ
	public String byteToHexString(byte ib) {
		char[] Digit = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A',
				'B', 'C', 'D', 'E', 'F' };
		char[] ob = new char[2];
		ob[0] = Digit[(ib >>> 4) & 0X0F];
		ob[1] = Digit[ib & 0X0F];
		String s = new String(ob);
		return s;
	}

	//byteתURLַ
	public String bytesToUrlString(byte[] b) {
		String st = "";
		for (int i = 0; i < b.length; i++) {
			byte bt = b[i];
			if ((bt >= 'a' && bt <= 'z') || (bt >= 'A' && bt <= 'Z')
					|| (bt >= '0' && bt <= '9')) {
				st += (char) bt;
			} else {
				st += "%" + byteToHexString(bt);
			}
		}
		return st;
	}

	//  btyeתURLַ ָʼλúͳ
	public String bytesToUrlString(byte[] b, int startPos, int length) {
		String st = "";
		for (int i = startPos; i < startPos + length; i++) {
			byte bt = b[i];
			if ((bt >= 'a' && bt <= 'z') || (bt >= 'A' && bt <= 'Z')
					|| (bt >= '0' && bt <= '9')) {
				st += (char) bt;
			} else {
				st += "%" + byteToHexString(bt);
			}
		}
		return st;
	}

	// ֽתΪString
	public String bytesToString(byte[] b) {
		StringBuffer result = new StringBuffer("");
		int length = b.length;
		for (int i = 0; i < length; i++) {
			result.append((char) (b[i] & 0xff));
		}
		return result.toString();
	}

	// ֽתΪStringָҪתĳ
	public String bytesToString(byte[] b, int length) {
		StringBuffer result = new StringBuffer("");
		for (int i = 0; i < length; i++) {
			result.append((char) (b[i] & 0xff));
		}
		return result.toString();
	}

	//ֽתΪStringָʼλúתĳ
	public String bytesToString(byte[] b, int startPos, int length) {
		StringBuffer result = new StringBuffer("");
		int newLenth = startPos + length;
		if (newLenth > b.length)
			newLenth = b.length;
		for (int i = startPos; i < newLenth; i++) {
			result.append((char) (b[i] & 0xff));
		}
		return result.toString();
	}

	// ֽתΪʮַ
	public String byteArrayToHexString(byte[] bytearray) {
		String strDigest = "";
		for (int i = 0; i < bytearray.length; i++) {
			strDigest += byteToHexString(bytearray[i]);
		}
		return strDigest;
	}

	//  ֽתΪʮַ
	public String byteArrayToHexString(byte[] bytearray, int length) {
		String strDigest = "";
		if (length > bytearray.length)
			length = bytearray.length;
		for (int i = 0; i < length; i++) {
			strDigest += byteToHexString(bytearray[i]);
		}
		return strDigest;
	}

	/**
	 * Little-EndianbyteתΪint
	 * @param b byte[]
	 * @return int
	 */
	public static int bytesLEToInt(byte[] t, int startLength) {
		int s = 0;
		byte[] b = new byte[4];
		System.arraycopy(t, startLength, b, 0, 4);
		for (int i = 0; i < 3; i++) {
			if (b[3 - i] >= 0) {
				s = s + b[3 - i];
			} else {
				s = s + 256 + b[3 - i];
			}
			s = s * 256;
		}
		if (b[0] >= 0) {
			s = s + b[0];
		} else {
			s = s + 256 + b[0];
		}
		return s;
	}
	
	/*
	public static void main(String[] args) {
		byte[] t = new byte[8];
		t[4] = 2;
		t[5] = 1;
		byte[] tt = new byte[4];
		tt[0] = 2;
		tt[1] = 1;
		System.out.println(bytesLEToInt(t, 4));

		//String digest = new Sha1().getDigestOfString(data.getBytes());
		//System.out.println(digest);
	}
	//*/
}
