package protocol;

import java.util.Random;

import bencode.ByteFormatter;
import configuration.*;

import message.*;
import component.*;

public class CheckStateTask implements Runnable {

	private final Component c;

	private ByteFormatter bf = new ByteFormatter();

	public CheckStateTask(Component c) {
		this.c = c;
	}

	private final void checkNeighborState() throws InterruptedException {

		sendKeepAlive();
		regularUnchoke();
		optimisticUnchoke();

	}

	private void sendKeepAlive() {

		synchronized (c.client) {

			int nSize = c.neighborList.size();
			for (int i = 0; i < nSize; i++) {
				Neighbor n = (Neighbor) c.neighborList.get(i);

				long currentTime = System.currentTimeMillis();
				boolean isSilence = currentTime - n.sendTime < Config.ALIVE_PERIOD / 4;
				boolean isActive = currentTime - n.recieveTime < Config.ALIVE_PERIOD;
				if (!isSilence && !isActive) {
					// System.out.println("send a keep alive message...");
					KeepAliveMsg kam = new KeepAliveMsg();
					n.sendingQueue.addLast(kam);
				}
			}
		}
	}

	private void regularUnchoke() throws InterruptedException {

		if (c.checkCycle % 2 != 0)
			return;

		synchronized (c.client) {

			int size = c.neighborList.size();

			// ðʿǰ
			for (int i = 0; i < size; i++) {
				for (int j = i + 1; j < size; j++) {
					Neighbor in = (Neighbor) c.neighborList.get(i);
					Neighbor jn = (Neighbor) c.neighborList.get(j);

					if (c.client.state == Client.SEED) {
						if (in.downloadRate < jn.downloadRate) {

							c.neighborList.set(i, jn);
							c.neighborList.set(j, in);

						}
					} else {
						if (in.uploadRate < jn.uploadRate) {

							c.neighborList.set(i, jn);
							c.neighborList.set(j, in);

						}
					}
				}
			}

			// ʼunchokechoke
			int unchokeNum = 0;
			for (int i = 0; i < size; i++) {

				// 0е
				Neighbor n = (Neighbor) c.neighborList.get(i);
				n.uploadRate = 0;
				n.downloadRate = 0;

				// unchokeھδﵽ
				if (unchokeNum < Config.REGULAR_UNCHOKE_NUM) {

					// ԭ״̬Ϊchoke
					if (n.amChoking == 1) {

						// unchokeϢע۴ھӶԼûȤ
						//System.out.println("send an unchoke message...");
						UnchokeMsg um = new UnchokeMsg();
						n.sendingQueue.addLast(um);
						n.amChoking = 0;

						/*
						 * ھӶԼȤunchokeNum1 ֤һunchokeھӶԼȤ
						 */
						if (n.peerInterested == 1) {
							unchokeNum++;
						}
					}
				} else {

					// ԭ״̬Ϊunchoke
					if (n.amChoking == 0) {
						//System.out.println("send a choke message...");
						ChokeMsg cm = new ChokeMsg();
						n.sendingQueue.addLast(cm);
						n.amChoking = 1;
					}
				}
			}
		}
	}

	private void optimisticUnchoke() throws InterruptedException {

		if (c.checkCycle % 6 != 0)
			return;

		synchronized (c.client) {

			int repeatNum = 1;
			Neighbor neighbor = null;
			int size = c.neighborList.size();
			for (int i = 0; i < size; i++) {

				Neighbor n = (Neighbor) c.neighborList.get(i);

				if (n.amChoking == 0)
					continue;
				if (n.peerInterested == 0)
					continue;

				Random r = new Random();
				int rr = r.nextInt(repeatNum);

				if (rr != 0)
					continue;
				neighbor = n;
				repeatNum++;
			}

			if (neighbor == null)
				return;
			UnchokeMsg um = new UnchokeMsg();
			neighbor.sendingQueue.addLast(um);
		}
	}

	private final void checkRequestState() {

		synchronized (c.client) {

			int size = c.pendingQueue.size();
			for (int i = 0; i < size; i++) {

				RequestMsg rm = (RequestMsg) c.pendingQueue.get(i);

				if (rm.requesterList.isEmpty())
					continue;

				long expirationTime = rm.creationTime + Config.MAX_WAITING_TIME;

				long currentTime = System.currentTimeMillis();
				if (currentTime < expirationTime)
					continue;

				Neighbor n = (Neighbor) rm.requesterList.getFirst();
				n.requestNum--;
				rm.requesterList.clear();

				int index = n.sendingQueue.indexOf(rm);
				if (index == -1)
					continue;

				n.sendingQueue.remove(index);
			}
		}
	}

	private final void checkTrackerState() throws InterruptedException {

		synchronized (c.client) {

			String event = "started";
			int numwant = Config.NUMWANT;
			if (c.client.state == Client.SEED) {
				event = "completed";
				numwant = 0;
			}

			String infoHash = bf.bytesToUrlString(c.fileInfo.infoHash);

			String parameters = "?info_hash=" + infoHash + "&peer_id="
					+ c.client.peerId + "&port=" + c.client.port
					+ "&natmapped=1&localip=" + c.client.ip
					+ "&port_type=wan&uploaded=" + c.client.uploaded
					+ "&downloaded=" + c.client.downloaded + "&left="
					+ c.client.left + "&numwant=" + numwant
					+ "&compact=1&no_peer_id=1&key=8144&event=" + event;

			int size = c.trackerList.size();
			for (int i = 0; i < size; i++) {
				Tracker t = (Tracker) c.trackerList.get(i);

				// /*

				if (c.neighborList.size() < Config.MIN_CONNECTION)
					t.nextReportTime -= (long) (t.interval * 1000 / 6);

				long currentTime = System.currentTimeMillis();
				if (currentTime < t.nextReportTime)
					continue;

				String announce = t.announce;

				RequestTrackerTask rtt = new RequestTrackerTask(c, announce,
						parameters);
				c.exec.execute(rtt);
				// */
			}
		}
	}

	public void run() {

		c.checkCycle++;

		try {
			checkNeighborState();
			checkRequestState();
			checkTrackerState();
		} catch (InterruptedException e) {
			e.printStackTrace();
			// Thread.currentThread().interrupt();
		}
	}
}
