package protocol;

import java.io.File;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.concurrent.*;

import bencode.BDecoder;
import bencode.ByteFormatter;
import configuration.Config;

import component.*;

public class DecodeTorrentTask implements Callable {
	
	private final String fileName;
	
	public DecodeTorrentTask(String fileName) {
		this.fileName = fileName;
	}
	
	private Component decodeTorrent() throws Exception {
		
		ByteFormatter bf = new ByteFormatter();
		
		BDecoder bdecoder = new BDecoder();
		bdecoder.print(new File("torrent//" + fileName));

		int pieceLen = bdecoder.pieceLength;
		int pieceNum = bdecoder.pieceHash.length / 20;
		System.out.println("pieceNum: " + pieceNum);
		
		int fSize = bdecoder.fileList.size();
		if (fSize == 0)
			return null;
		
		int[] lenList = new int[fSize];
		File[] fileList = new File[fSize];
		RandomAccessFile[] raf = new RandomAccessFile[fSize];

		String name = bdecoder.name;
		long fileLen = 0;
		String dir = "download//" + name;
		for (int i = 0; i < fSize; i++) {
				
			BDecoder.Files f = (BDecoder.Files) bdecoder.fileList.get(i);
				
			fileLen += f.length;
			lenList[i] = f.length;
			
			if (fSize !=  1) {
				File file = new File(dir);
				if (!(file.exists()))file.mkdirs();
				String path = f.path;
				fileList[i] = new File(dir + "//" + path);
			} else {
				fileList[i] = new File(dir);
			}
			
			raf[i] = new RandomAccessFile(fileList[i], "rw");
			lenList[i] = f.length;
				
			System.out.println(i + ", len: " + f.length);
			System.out.println(i + ", path: " + f.path);
		}
			
		System.out.println("fileLen: " + fileLen);

		int blockNum = (int) (fileLen / Config.BLOCK);
		int lastBlockLen = (int) (fileLen % Config.BLOCK);
		if (lastBlockLen != 0) {
			blockNum++;
		}
		System.out.println("blockNum: " + blockNum);
		
		byte[] infoHash = (byte[])bdecoder.infoHash.clone();
		Client client = new Client(pieceNum);
		client.left = fileLen;
		FileInfo fileInfo = new FileInfo(blockNum, pieceLen, pieceNum, fileLen,
				lenList, infoHash);

		Component c = new Component(client, fileInfo, raf);
		
		ArrayList announceList = bdecoder.announceList;
		int aSize = announceList.size();
		for(int i = 0; i < aSize; i++) {
			String announce = bf.bytesToString((byte[])announceList.get(i));
			Tracker tracker = new Tracker(announce);
			c.trackerList.add(tracker);
		}
		
		return c;
	}
	
	public Object call() {
		
		try {
			Component c = decodeTorrent();
			return c;
		} catch(Exception e) {
			e.printStackTrace();
			return null;
		}
	}
}
