package protocol;

import bencode.ByteFormatter;
import message.*;
import component.*;
import configuration.*;

public class HandleMessageTask implements Runnable {

	private final Component c;

	private ByteFormatter bf = new ByteFormatter();

	public HandleMessageTask(Component c) {

		this.c = c;
		//System.out.println("create a house constructor task...");
	}

	private void handleMsg(Message msg) throws Exception {

		switch (msg.msgId) {
		case Message.CHOKE_MSG:
			//System.out.println("response a choke message...");
			responseChokeMsg((ChokeMsg) msg);
			break;
		case Message.UNCHOKE_MSG:
			//System.out.println("responses an unchoke message...");
			responseUnchokeMsg((UnchokeMsg) msg);
			break;
		case Message.INTERESTED_MSG:
			//System.out.println("response an interested message...");
			responseInterestedMsg((InterestedMsg) msg);
			break;
		case Message.NOT_INTERESTED_MSG:
			//System.out.println("response a not interested message...");
			responseNotInterestedMsg((NotInterestedMsg) msg);
			break;
		case Message.HAVE_MSG:
			//System.out.println("response a have message...");
			responseHaveMsg((HaveMsg) msg);
			break;
		case Message.BITFIELD_MSG:
			System.out.println("response a bitfield message...");
			responseBitfieldMsg((BitfieldMsg) msg);
			break;
		case Message.REQUEST_MSG:
			//System.out.println("responses a request message...");
			responseRequestMsg((RequestMsg) msg);
			break;
		case Message.PIECE_MSG:
			//System.out.println("response a piece message...");
			responsePieceMsg((PieceMsg) msg);
			break;
		case Message.CANCEL_MSG:
			//System.out.println("response a cancel message...");
			responseCancelMsg((CancelMsg) msg);
			break;
		case Message.KEEEP_ALIVE_MSG:
			//System.out.println("response a keep alive message...");
			responseKeepAliveMsg((KeepAliveMsg) msg);
			break;
		case Message.HANDSHAKE:
			//System.out.println("response a handshake...");
			responseHandshake((Handshake) msg);
			break;

		default:

		}
	}

	/**
	 * ӦChokeϢpeerChoking˻ػδյRequestϢ
	 * @param cm ChokeϢ
	 */
	private void responseChokeMsg(ChokeMsg cm) {

		synchronized (c.client) {

			int nSize = c.neighborList.size();
			for (int i = 0; i < nSize; i++) {
				Neighbor n = (Neighbor) c.neighborList.get(i);
				if (!n.address.equals(cm.address))
					continue;

				// ýʱ
				n.recieveTime = System.currentTimeMillis();

				// peerChoking
				n.peerChoking = 1;

				break;
			}
		}
	}

	/**
	 * ӦUnchokeϢpeerChokingRequestϢ
	 * @param um UnconnectϢ
	 * @throws InterruptedException
	 */
	private void responseUnchokeMsg(UnchokeMsg um) throws InterruptedException {

		// ʹͳһ
		synchronized (c.client) {

			int size = c.neighborList.size();
			for (int i = 0; i < size; i++) {
				Neighbor n = (Neighbor) c.neighborList.get(i);
				if (!n.address.equals(um.address))
					continue;

				// ýʱ
				n.recieveTime = System.currentTimeMillis();

				// peerChoking
				n.peerChoking = 0;

				// RequestϢΪܣԻRequestϢ
				// ΪõһpieceStrict Rarest-first
				strictRarestFirst(n);

				// Local Rarest-first
				localRarestFirst(n);

				break;
			}
		}
	}

	/**
	 * ϸȣּھõһƬΪδҴعеpiece
	 * @param n ھϢ
	 * @return Ƿɹ
	 * @throws InterruptedException
	 */
	private final void strictRarestFirst(Neighbor n)
			throws InterruptedException {

		int pSize = c.pendingQueue.size();
		for (int i = 0; i < pSize; i++) {

			if (n.requestNum >= Config.MAX_REQUEST_NUM)
				break;

			RequestMsg rm = (RequestMsg) c.pendingQueue.get(i);

			// Ѿ󣬲
			if (!rm.requesterList.isEmpty())
				continue;

			int index = bf.byteArrayToInt(rm.msgContext, 5);
			int bitIndex = index / 8;
			int bitOffset = index % 8;

			// ھûдblock
			if ((n.bitfield[bitIndex] & (128 >> bitOffset)) == 0)
				continue;

			rm.creationTime = System.currentTimeMillis();
			rm.requesterList.add(n);
			n.sendingQueue.addLast(rm);
			n.requestNum++;
			//System.out.println("add a new request through the strict rarest first...");

		}
	}

	/**
	 * ֲȣؾֲٵpiece
	 * @param n ھϢ
	 * @throws InterruptedException
	 */
	private final void localRarestFirst(Neighbor n) throws InterruptedException {

		if (n.requestNum >= Config.MAX_REQUEST_NUM)
			return;

		int rarestIndex = c.client.getRarestIndex(n.bitfield);

		// Լδpieceھ޿piece
		if (rarestIndex == -1) {
			//System.out.println("the neighbor does not have the piece that the client wants to download....");
			return;
		}

		// ҵpiece
		requestRarest(rarestIndex, n);
		//System.out.println("add a new request for piece " + rarestIndex + " through the local rarest first...");
	}

	/**
	 * ɲRequestϢδ͵RequestϢҵpendingQueue
	 * @param rarestIndex pieceindexn ھϢ
	 * @throws InterruptedException
	 */
	private final void requestRarest(int rarestIndex, Neighbor n)
			throws InterruptedException {

		int lastPieceIndex = c.fileInfo.pieceNum - 1;

		// ׼pieceļһpiece
		if (rarestIndex == lastPieceIndex) {

			int lastBlockIndex = c.fileInfo.blockNumInLastPiece - 1;
			for (int i = 0; i < c.fileInfo.blockNumInLastPiece; i++) {
				byte[] context = new byte[12];
				bf.intToByteArray(rarestIndex, context, 0);
				int blockIndex = i * c.fileInfo.blockLen;
				bf.intToByteArray(blockIndex, context, 4);
				if (i == lastBlockIndex) {
					bf.intToByteArray(c.fileInfo.lastBlockLen, context, 8);
				} else {
					bf.intToByteArray(c.fileInfo.blockLen, context, 8);
				}

				RequestMsg rm = new RequestMsg(context);

				// RequestϢ
				c.pendingQueue.addLast(rm);

				if (n.requestNum >= Config.MAX_REQUEST_NUM)
					continue;
				
				rm.requesterList.add(n);
				n.sendingQueue.add(rm);
				n.requestNum++;
			}

		} else {// ׼pieceļһpiece

			for (int i = 0; i < c.fileInfo.blockNumInPiece; i++) {
				byte[] context = new byte[12];
				bf.intToByteArray(rarestIndex, context, 0);
				int blockIndex = i * c.fileInfo.blockLen;
				bf.intToByteArray(blockIndex, context, 4);
				bf.intToByteArray(c.fileInfo.blockLen, context, 8);

				RequestMsg rm = new RequestMsg(context);

				// RequestϢ
				c.pendingQueue.addLast(rm);

				if (n.requestNum >= Config.MAX_REQUEST_NUM)
					continue;
				
				rm.requesterList.add(n);
				n.sendingQueue.add(rm);
				n.requestNum++;
			}
		}
	}

	/**
	 * ӦInterestedMsgϢpeerInterested
	 * @param im InterestedϢ
	 */
	private void responseInterestedMsg(InterestedMsg im) {

		// ʹͳһ
		synchronized (c.client) {

			int size = c.neighborList.size();
			for (int i = 0; i < size; i++) {
				Neighbor n = (Neighbor) c.neighborList.get(i);
				if (!n.address.equals(im.address))
					continue;

				// ýʱ
				n.recieveTime = System.currentTimeMillis();

				// peerInterested
				n.peerInterested = 1;
				break;
			}
		}
	}

	/**
	 * ӦNotInterestedϢpeerInterested
	 * @param nim NotInterestedϢ
	 */
	private void responseNotInterestedMsg(NotInterestedMsg nim) {

		// ʹͳһ
		synchronized (c.client) {

			int size = c.neighborList.size();
			for (int i = 0; i < size; i++) {
				Neighbor n = (Neighbor) c.neighborList.get(i);
				if (!n.address.equals(nim.address))
					continue;

				// ýʱ
				n.recieveTime = System.currentTimeMillis();

				// peerInterested
				n.peerInterested = 0;
				break;
			}
		}
	}

	/**
	 * ӦHaveϢھbitfieldԼbitMap 
	 * ΪȤInterestedϢ
	 * @param msg HaveϢ
	 * @throws InterruptedException
	 */
	private void responseHaveMsg(HaveMsg hm) throws InterruptedException {

		// ʹͳһ
		synchronized (c.client) {

			int pieceIndex = bf.byteArrayToInt(hm.msgContext, 5);

			int size = c.neighborList.size();
			for (int i = 0; i < size; i++) {
				Neighbor n = (Neighbor) c.neighborList.get(i);
				if (!n.address.equals(hm.address))
					continue;

				// ýʱ					
				n.recieveTime = System.currentTimeMillis();

				// ھbitfield
				n.setBitfield(pieceIndex);
				// ԼbitMap
				c.client.bitMap[pieceIndex]++;

				// ԭԼھӲȤڱΪȤ
				if (n.becomeInterested(c.client.bitfield)) {
					// InterestedϢ
					InterestedMsg im = new InterestedMsg();
					n.sendingQueue.addLast(im);
				}
				break;
			}
		}
	}

	/**
	 * ӦBitfieldϢھӵbitfieldԼbitMap
	 * ȤInterestedϢNotInterestedϢ
	 * @param bm BitfieldϢ
	 * @throws InterruptedException
	 */
	private void responseBitfieldMsg(BitfieldMsg bm)
			throws InterruptedException {

		// ʹͳһ
		synchronized (c.client) {

			int size = c.neighborList.size();
			for (int i = 0; i < size; i++) {
				Neighbor n = (Neighbor) c.neighborList.get(i);
				if (!n.address.equals(bm.address))
					continue;

				// ýʱ
				n.recieveTime = System.currentTimeMillis();

				// ھӵbitfield
				//System.out.println(n.bitfield.length);
				//System.out.println(bm.msgContext.length - 5);

				System.arraycopy(bm.msgContext, 5, n.bitfield, 0,
						n.bitfield.length);
				// ԼbitMap
				c.client.updateBitMap(n.bitfield);

				// ԼھӸȤ
				if (n.becomeInterested(c.client.bitfield)) {

					//System.out.println("return an Interested message...");
					InterestedMsg im = new InterestedMsg();
					n.sendingQueue.addLast(im);
				} else {
					//System.out.println("bt client is not interested in this neighbor...");
				}

				break;
			}
		}
	}

	/**
	 * ӦRequestϢPieceϢ
	 * @param rm RequestϢ
	 * @throws Exception
	 */
	private void responseRequestMsg(RequestMsg rm) throws Exception {

		synchronized (c.client) {

			int size = c.neighborList.size();
			for (int i = 0; i < size; i++) {
				Neighbor n = (Neighbor) c.neighborList.get(i);
				if (!n.address.equals(rm.address))
					continue;

				// ýʱ
				n.recieveTime = System.currentTimeMillis();

				// Լchokeھӣֱӷ
				if (n.amChoking == 1)
					break;

				// Լھӣھٶȼ1
				n.downloadRate++;

				// ȴcacheжȡ
				if (readCache(rm, n))
					break;

				// ɹȡļ
				readFile(rm, n);
				break;
			}
		}
	}

	/**
	 * ȡbufferPieceϢ
	 * @param rm RequestϢ
	 * @return ϢǷͳɹ
	 * @throws InterruptedException
	 */
	private boolean readCache(RequestMsg rm, Neighbor n)
			throws InterruptedException {

		int index = bf.byteArrayToInt(rm.msgContext, 5);
		int begin = bf.byteArrayToInt(rm.msgContext, 9);
		int length = bf.byteArrayToInt(rm.msgContext, 13);

		// System.out.println("recieve a request message (piece " + index + ") from the neighbor............");

		synchronized (c.writingQueue) {

			int size = c.writingQueue.size();
			for (int i = 0; i < size; i++) {
				Piece p = (Piece) c.writingQueue.get(i);
				if (p.pieceIndex != index)
					continue;

				if (!p.isPieceCompleted())
					continue;

				byte[] context = new byte[8 + length];

				// RequestϢindexbegincontext
				System.arraycopy(rm.msgContext, 5, context, 0, 8);
				// blockcontext
				System.arraycopy(p.pieceContext, begin, context, 8, length);

				// ϢӵsendingQueue
				PieceMsg pm = new PieceMsg(context);
				n.sendingQueue.addLast(pm);

				c.client.uploaded += length;
				return true;
			}
		}

		int rSize = c.readingQueue.size();
		for (int i = 0; i < rSize; i++) {
			Piece p = (Piece) c.readingQueue.get(i);
			if (p.pieceIndex != index)
				continue;

			byte[] context = new byte[8 + length];

			// RequestϢindexbegincontext
			System.arraycopy(rm.msgContext, 5, context, 0, 8);
			// blockcontext
			System.arraycopy(p.pieceContext, begin, context, 8, length);

			// ϢӵsendingQueue
			PieceMsg pm = new PieceMsg(context);
			n.sendingQueue.addLast(pm);

			c.client.uploaded += length;
			return true;
		}

		return false;
	}

	/**
	 * ȡ汻pieceȻPieceϢ
	 * @param rm RequestϢ
	 * @return ضȡPiece
	 * @throws Exception
	 */
	private final void readFile(RequestMsg rm, Neighbor n) throws Exception {

		int index = bf.byteArrayToInt(rm.msgContext, 5);
		int begin = bf.byteArrayToInt(rm.msgContext, 9);
		int length = bf.byteArrayToInt(rm.msgContext, 13);

		int startPos = index * c.fileInfo.pieceLen;

		int lastPieceIndex = c.fileInfo.pieceNum - 1;
		int pieceLen = 0;
		int blockNumInPiece = 0;

		// һpiece
		if (index == lastPieceIndex) {
			pieceLen = c.fileInfo.lastPieceLen;
			blockNumInPiece = c.fileInfo.blockNumInLastPiece;
		} else {// һpiece
			pieceLen = c.fileInfo.pieceLen;
			blockNumInPiece = c.fileInfo.blockNumInPiece;
		}

		int endPos = startPos + pieceLen;

		byte[] piece = new byte[pieceLen];
		int leftLen = pieceLen; // ʣpieceȡ

		int currentPos = 0; // Ѿλļλ
		int nextPos = 0; // һζλļλ

		synchronized (c.randomAccessFile) {

			int fileNum = c.fileInfo.lenList.length;
			for (int i = 0; i < fileNum; i++) {
				nextPos += c.fileInfo.lenList[i];
				if (startPos < nextPos) {
					if (endPos > nextPos) { // ֱʼַָļ,ʼд
						c.randomAccessFile[i].seek(startPos - currentPos);
						c.randomAccessFile[i].read(piece, pieceLen - leftLen,
								nextPos - startPos);
						leftLen = startPos + leftLen - nextPos;
						startPos = nextPos;
					} else { // Ҫļȫļ
						c.randomAccessFile[i].seek(startPos - currentPos);
						c.randomAccessFile[i].read(piece, pieceLen - leftLen,
								leftLen);
						break;
					}
				}
				currentPos += c.fileInfo.lenList[i];
			}
		}

		// PieceϢ
		byte[] context = new byte[8 + length];
		// RequestϢindexbegincontext
		System.arraycopy(rm.msgContext, 5, context, 0, 8);
		// blockcontext
		Piece p = new Piece(index, blockNumInPiece, piece);
		System.arraycopy(p.pieceContext, begin, context, 8, length);
		PieceMsg pm = new PieceMsg(context);
		n.sendingQueue.addLast(pm);

		c.client.uploaded += length;

		// ȡpiece
		c.readingQueue.addLast(p);
		if (!(c.readingQueue.size() < Config.MAX_READ_CACHE))
			c.readingQueue.removeFirst();

	}

	/**
	 * ӦPieceϢRequestϢ
	 * @param msg PieceϢ
	 * @throws InterruptedException
	 */
	private void responsePieceMsg(PieceMsg pm) throws InterruptedException {

		// ѾSeedģʽֱӷ
		if (c.client.state == Client.SEED)
			return;

		// ɾRequestϢµRequestϢ
		addRequest(pm);

		// PieceϢеblock
		int index = cacheBlock(pm);
		if (index == -1)
			return;

		addHave(index);
		//System.out.println("broadcast a have message.....");

		if (!c.client.isNewSeed())
			return;

		c.endTime = System.currentTimeMillis();
		double downloadTime = (double) (c.endTime - c.startTime) / 1000;
		c.write(downloadTime + "");

		System.out.println("This peer enters the seed mode...");
		reportTracker();
	}

	private void addRequest(PieceMsg pm) throws InterruptedException {

		synchronized (c.client) {
			
			int pSize = c.pendingQueue.size();
			for (int i = 0; i < pSize; i++) {
				RequestMsg rm = (RequestMsg) c.pendingQueue.get(i);

				if (!rm.equals(pm.msgContext))
					continue;

				// ɾӦRequestϢ
				c.pendingQueue.remove(i);
				
				// requesterListΪգʱ
				if (rm.requesterList.isEmpty())
					break;

				Neighbor n = (Neighbor) rm.requesterList.getFirst();

				// ýʱ
				n.recieveTime = System.currentTimeMillis();

				// Լھأھϴٶȼ1choke㷨
				n.uploadRate++;
				
				// ɾŶӵRequest
				n.requestNum--;
				
				if (n.peerChoking == 1)
					return;

				// RequestϢΪܣԻRequestϢ
				// ΪõһpieceStrict Rarest-first
				strictRarestFirst(n);

				// Local Rarest-first
				localRarestFirst(n);
					
				return;
			}
			
			// ʱδ·䣬ǰһھӣrequesterListΪգ
			int nSize = c.neighborList.size();
			for (int j = 0; j < nSize; j++) {
				Neighbor n = (Neighbor) c.neighborList.get(j);

				if (!n.address.equals(pm.address))
					continue;

				// ýʱ
				n.recieveTime = System.currentTimeMillis();

				// Լھأھϴٶȼ1choke㷨
				n.uploadRate++;
				
				if (n.peerChoking == 1)
					break;

				// RequestϢΪܣԻRequestϢ
				// ΪõһpieceStrict Rarest-first
				strictRarestFirst(n);

				// Local Rarest-first
				localRarestFirst(n);

				break;
			}
		}
	}

	/**
	 * PieceϢеblock
	 * @param pm PieceϢ
	 * @throws InterruptedException
	 */
	private int cacheBlock(PieceMsg pm) {

		int len = bf.byteArrayToInt(pm.msgContext, 0);
		int index = bf.byteArrayToInt(pm.msgContext, 5);
		//System.out.println("delete the request message for piece " + index + " from the pinding queue....");
		int begin = bf.byteArrayToInt(pm.msgContext, 9);

		//System.out.println("piece: " + index + ".....");

		int length = len - 9;
		int blockIndex = begin / c.fileInfo.blockLen;

		// סwritinhQueuecache
		synchronized (c.writingQueue) {

			int wSize = c.writingQueue.size();
			for (int i = 0; i < wSize; i++) {
				Piece p = (Piece) c.writingQueue.get(i);

				// writeBufferQueueҵӦpiece
				if (p.pieceIndex != index)
					continue;

				// Ѿ棬ֱӷ
				if (p.blockMap[blockIndex] == 1)
					return -1;

				// blockMapӦblockΪ1
				p.blockMap[blockIndex] = 1;
				c.client.left -= length;
				c.client.downloaded += length;
				System.out.println("left: " + c.client.left + "....");

				// PieceϢеblock
				System.arraycopy(pm.msgContext, 13, p.pieceContext, begin,
						length);

				if (p.isPieceCompleted())
					return index;
				else
					return -1;
			}

			// ûҵȴһPieceȻ󻺴
			int lastPieceIndex = c.fileInfo.pieceNum - 1;

			int blockNumInPiece = 0;
			int pieceLen = 0;
			if (index == lastPieceIndex) {
				blockNumInPiece = c.fileInfo.blockNumInLastPiece;
				pieceLen = c.fileInfo.lastPieceLen;
			} else {
				blockNumInPiece = c.fileInfo.blockNumInPiece;
				pieceLen = c.fileInfo.pieceLen;
			}

			Piece p = new Piece(index, blockNumInPiece, new byte[pieceLen]);
			p.blockMap[blockIndex] = 1;
			c.client.left -= length;
			c.client.downloaded += length;
			System.arraycopy(pm.msgContext, 13, p.pieceContext, begin, length);
			c.writingQueue.addFirst(p);

			if (p.isPieceCompleted())
				return index;
			else
				return -1;
		}
	}

	private final void addHave(int index) {

		synchronized (c.client) {

			// ԼbitMapbitfield
			c.client.setBitfield(index);

			// 㲥HaveϢ
			byte[] context = new byte[4];
			bf.intToByteArray(index, context, 0);
			HaveMsg hm = new HaveMsg(context);

			// ھӣʧȥȤNotInterestedϢ
			int size = c.neighborList.size();
			for (int j = 0; j < size; j++) {
				Neighbor n = (Neighbor) c.neighborList.get(j);
				n.sendingQueue.addLast(hm);

				if (n.becomeNotInterested(c.client.bitfield)) {
					NotInterestedMsg nim = new NotInterestedMsg();
					n.sendingQueue.addLast(nim);
				}
			}
		}
	}

	private final void reportTracker() {

		synchronized (c.client) {

			String infoHash = bf.bytesToUrlString(c.fileInfo.infoHash);

			String parameters = "?info_hash=" + infoHash + "&peer_id="
					+ c.client.peerId + "&port=" + c.client.port
					+ "&natmapped=1&localip=" + c.client.ip
					+ "&port_type=wan&uploaded=" + c.client.uploaded
					+ "&downloaded=" + c.client.downloaded + "&left="
					+ c.client.left + "&numwant=" + Config.NUMWANT
					+ "&compact=1&no_peer_id=1&key=8144&event=completed";

			int tSize = c.trackerList.size();
			for (int i = 0; i < tSize; i++) {
				Tracker t = (Tracker) c.trackerList.get(i);

				RequestTrackerTask rtt = new RequestTrackerTask(c, t.announce,
						parameters);
				c.exec.execute(rtt);
			}
		}
	}

	/**
	 * ӦKeepAliveϢkeepAliveTimeΪǰϵͳʱ
	 * @param kam KeepAliveϢ
	 */
	private void responseKeepAliveMsg(KeepAliveMsg kam) {

		synchronized (c.client) {

			int size = c.neighborList.size();
			for (int i = 0; i < size; i++) {
				Neighbor n = (Neighbor) c.neighborList.get(i);
				if (!n.address.equals(kam.address))
					continue;

				// ýʱ					
				n.recieveTime = System.currentTimeMillis();
				break;
			}
		}
	}

	/**
	 * ӦHandshakeϢ
	 * @param hs HandshakeϢ
	 * @throws InterruptedException
	 */
	private void responseHandshake(Handshake hs) throws InterruptedException {

		synchronized (c.client) {

			int size = c.neighborList.size();
			for (int i = 0; i < size; i++) {
				Neighbor n = (Neighbor) c.neighborList.get(i);

				// ھѾʼ˵ڵ
				if (!n.address.equals(hs.address))
					continue;

				// ýʱ				
				n.recieveTime = System.currentTimeMillis();

				// ӣHandshake
				if (!n.isOutgoing) {
					byte[] infoHash = c.fileInfo.infoHash;
					byte[] peerId = c.client.peerId.getBytes();

					byte[] context = new byte[40];
					System.arraycopy(infoHash, 0, context, 0, 20);
					System.arraycopy(peerId, 0, context, 20, 20);

					Handshake nhs = new Handshake(context);
					n.sendingQueue.addLast(nhs);
				}

				// BitfieldϢ˳
				byte[] bitfield = (byte[]) c.client.bitfield.clone();
				BitfieldMsg bm = new BitfieldMsg(bitfield);
				n.sendingQueue.addLast(bm);

				//System.out.println("return a bitfield message...");
				break;
			}
		}
	}

	/**
	 * ӦCancelϢ
	 * @param cm CancelϢ
	 */
	private void responseCancelMsg(CancelMsg cm) {

		synchronized (c.client) {

			int nSize = c.neighborList.size();
			for (int i = 0; i < nSize; i++) {
				Neighbor n = (Neighbor) c.neighborList.get(i);
				if (!n.address.equals(cm.address))
					continue;

				// ýʱ
				n.recieveTime = System.currentTimeMillis();

				// ȡCancelϢӦPieceϢ
				cancelRequest(n, cm);

				break;
			}
		}
	}

	public void cancelRequest(Neighbor n, CancelMsg cm) {

		int sSize = n.sendingQueue.size();
		for (int j = 0; j < sSize; j++) {
			Message msg = (Message) n.sendingQueue.get(j);
			if (msg.msgId != Message.PIECE_MSG)
				continue;
			if (!cm.equals(msg.msgContext))
				continue;
			n.sendingQueue.remove(j);
			break;
		}
	}

	public void run() {

		try {
			System.out.println("start a handle message task...");
			while (true) {
				handleMsg((Message) c.receivingQueue.take());
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}