package protocol;

import java.net.*;
import java.nio.*;
import java.nio.channels.*;
import java.io.*;
import java.util.*;

import configuration.*;
import component.*;
import message.*;
import bencode.*;

public class ReadMessageTask extends Thread {
	
	private Component c;
    
    private Selector selector;
     
	private final ByteFormatter bf;
    
	public ReadMessageTask(Component c){
        
		this.c = c;
		this.bf = new ByteFormatter();
        
        try {
        	this.selector = Selector.open();
        } catch (IOException e) {
            e.printStackTrace();
        }
	}
	
	private void readMsg() {

		try {
			int numKeys = selector.select(30);
			if (numKeys == 0)
				return;
		} catch (IOException e) {
			e.printStackTrace();
			return;
		}

		Set readyKeys = selector.selectedKeys();
		Iterator iter = readyKeys.iterator();

		_OUT: while (iter.hasNext()) {

			SelectionKey key = (SelectionKey) iter.next();
			iter.remove();

			SocketChannel client = (SocketChannel) key.channel();
			InetSocketAddress address = (InetSocketAddress) client.socket()
					.getRemoteSocketAddress();

			ByteBuffer buffer = (ByteBuffer) key.attachment();

			try {

				buffer.limit(1);
				int pos = 0;
				while (pos == 0) {
					pos += client.read(buffer);
				}

				if (pos == -1) {
					System.out.println("channel has been closed...");
					client.close();
					key.cancel();
					continue _OUT;
				}

				int msgLen = 0;
				if (buffer.get(0) == (byte) 0X13) {
					msgLen = 68;
				} else {
					buffer.limit(4);
					while (pos < buffer.limit()) {
						buffer.position(pos);
						pos += client.read(buffer);
					}

					byte[] ba = new byte[4];
					for (int i = 0; i < 4; i++) {
						ba[i] = buffer.get(i);
					}
					int len = bf.byteArrayToInt(ba, 0);
					msgLen = 4 + len;
				}

				buffer.limit(msgLen);
				// System.out.println("limit: " + buffer.limit());
				while (pos < buffer.limit()) {
					buffer.position(pos);
					pos += client.read(buffer);

				}

				// System.out.println("pos: " + pos);
				// System.out.println("receive a message that has a length " +
				// msgLen + "....");

				byte[] msgContext = new byte[(int) msgLen];
				for (int j = 0; j < msgLen; j++) {
					msgContext[j] = buffer.get(j);
				}

				Message msg = receiveMsg(msgContext, address);
				c.receivingQueue.put(msg);

				buffer.clear();

			} catch (Exception e) {
				e.printStackTrace();
				key.cancel();
			}
		}
	}
	
	private final byte getMsgId(byte[] msgContext) {

		if (msgContext[0] == (byte) 0X13)
			return Message.HANDSHAKE;

		if (msgContext.length == 4) {
			return Message.KEEEP_ALIVE_MSG;
		}

		return msgContext[4];
	}

	private final Message receiveMsg(byte[] msgContext, InetSocketAddress address) {

		byte msgId = getMsgId(msgContext);
		switch (msgId) {
		case Message.HANDSHAKE:
			//System.out.println("receive a handshake...");
			Handshake handshake = new Handshake(msgContext, address);
			return handshake;
		case Message.KEEEP_ALIVE_MSG:// յkeeepAliveMsg
			//System.out.println("recieve a keep-alive message...");
			KeepAliveMsg keepAliveMsg = new KeepAliveMsg(address);
			return keepAliveMsg;
		case Message.CHOKE_MSG: // յchokeMsg
			//System.out.println("recieve a choke message...");
			ChokeMsg chokeMsg = new ChokeMsg(address);
			return chokeMsg;
		case Message.UNCHOKE_MSG: // յUnChokeMsg
			//System.out.println("recieve an unchoke message...");
			UnchokeMsg unchokeMsg = new UnchokeMsg(address);
			return unchokeMsg;
		case Message.INTERESTED_MSG: // յInterestedMsg
			//System.out.println("recieve an interested message...");
			InterestedMsg interestedMsg = new InterestedMsg(address);
			return interestedMsg;
		case Message.NOT_INTERESTED_MSG:// յNotInterestedMsg
			//System.out.println("recieve a not interested message...");
			NotInterestedMsg notInterestedMsg = new NotInterestedMsg(address);
			return notInterestedMsg;
		case Message.HAVE_MSG: // յHaveMsg
			//System.out.println("recieve a have message...");
			HaveMsg haveMsg = new HaveMsg(msgContext, address);
			return haveMsg;
		case Message.BITFIELD_MSG: // յbitFieldMsg
			//System.out.println("recieve a bitfiled message...");
			BitfieldMsg bitfieldMsg = new BitfieldMsg(msgContext, address);
			return bitfieldMsg;
		case Message.REQUEST_MSG: // յRequestMsg
			//System.out.println("recieve a request message...");
			RequestMsg requestMsg = new RequestMsg(msgContext, address);
			return requestMsg;
		case Message.PIECE_MSG: // յPieceMsg
			//System.out.println("recieve a piece message...");
			PieceMsg pieceMsg = new PieceMsg(msgContext, address);
			return pieceMsg;
		case Message.CANCEL_MSG: // յcancelMsg
			//System.out.println("recieve a cancel message...");
			CancelMsg cancelMsg = new CancelMsg(msgContext, address);
			return cancelMsg;
		case Message.PORT_MSG: // յportMsg
			return null;

		default:
			return null;
		}
	}

	
    private void registerKey() {

		while (!c.readers.isEmpty()) {

			SocketChannel client = (SocketChannel) c.readers.poll();
			try {
				client.configureBlocking(false);
				client.register(selector, SelectionKey.OP_READ, ByteBuffer
						.allocate(Config.BUFFER));
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

    public void run() {

		while (true) {

			registerKey();
			readMsg();
			
			/*
			try {
				Thread.sleep(30);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			//*/
		}
	}
}
