package protocol;

import java.io.BufferedInputStream;
import java.net.HttpURLConnection;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.URL;
import java.net.UnknownHostException;

import component.*;
import bencode.*;

public class RequestTrackerTask implements Runnable {

	private final int bufferSize = 100 * 1024;

	private final int connectTimeout = 10000;

	private final int readTimeout = 10000;

	private final String announce;

	private final String parameters;

	private final Component c;

	public RequestTrackerTask(Component c, String announce, String parameters) {

		this.announce = announce;
		this.parameters = parameters;
		this.c = c;
	}

	private final void requestTracker() throws Exception {

		URL url = new URL(announce + parameters);
		System.out.println(url.toString());
		HttpURLConnection link = (HttpURLConnection) url.openConnection();

		link.setConnectTimeout(connectTimeout);
		link.setReadTimeout(readTimeout);
		link.setDoOutput(true);
		link.connect();

		BufferedInputStream bis = new BufferedInputStream(link.getInputStream());

		byte[] buffer = new byte[bufferSize];
		int count = bis.read(buffer);
		//System.out.println("count: " + count);

		bis.close();
		link.disconnect();

		if (count > 0) {

			//System.out.println("receive a response from the tracker...");
			byte[] response = new byte[count];
			System.arraycopy(buffer, 0, response, 0, count);

			handleResponse(response);
		}
	}

	private final void handleResponse(byte[] response)
			throws UnknownHostException {

		BDecoder b = new BDecoder();
		b.print(response);

		InetAddress localIp = InetAddress.getLocalHost();

		synchronized (c.client) {

			int pSize = b.peerList.size();
			for (int j = 0; j < pSize; j++) {

				BDecoder.Peers p = (BDecoder.Peers) b.peerList.get(j);
				InetSocketAddress address = new InetSocketAddress(p.ip, p.port);
				
				///*
				if (c.addressQueue.contains(address))
					continue;
				//*/	
				
				///*
				InetAddress ip = address.getAddress();
				if (ip.equals(localIp) && address.getPort() == c.client.port)
					continue;	
				//*/
				
				c.addressQueue.addLast(address);

				System.out.println(p.ip + ":" + p.port);
			}

			int size = c.trackerList.size();
			for (int i = 0; i < size; i++) {
				Tracker t = (Tracker) c.trackerList.get(i);
				if (!t.announce.equals(announce))
					continue;

				System.out.println("set tracker info...");
				t.complete = b.complete;
				t.incomplete = b.incomplete;
				t.interval = b.interval;
				t.nextReportTime = System.currentTimeMillis() + t.interval
						* 1000;
				
				//System.out.println("interval: " + t.interval);
			}
		}
	}

	private final void delTracker() {

		synchronized (c.client) {

			int size = c.trackerList.size();
			for (int i = 0; i < size; i++) {
				Tracker t = (Tracker) c.trackerList.get(i);
				if (!t.announce.equals(announce))
					continue;
				c.trackerList.remove(i);
				break;
			}
		}
	}

	public void run() {

		try {
			requestTracker();
		} catch (Exception e) {
			e.printStackTrace();

			System.out.println("delete the Tracker...");
			delTracker();
		}
	}
}
